#!/usr/bin/env python
# -*- coding: utf-8 -*-

# --------------------------------------------------------------------
# The MIT License (MIT)
#
# Copyright (c) 2014 Jonathan Labéjof <jonathan.labejof@gmail.com>
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
# --------------------------------------------------------------------

"""conf file driver UTs."""

from unittest import main

from ..base import FileConfDriver, CONF_DIRS
from ...test.base import ConfDriverTest
from ....model.param import Parameter

from pickle import load, dump

from os import remove
from os.path import exists, join


class TestFileConfDriver(FileConfDriver):
    """Configuration file driver for test."""

    def resource(self):

        return {}

    def _cnames(self, resource):

        return resource.keys()

    def _params(self, resource, cname):

        result = [
            Parameter(pname, svalue=resource[cname][pname])
            for pname in resource[cname]
        ]

        return result

    def _pathresource(self, rscpath):

        result = None

        with open(rscpath, 'rb') as handle:

            result = load(handle)

        return result

    def _setconf(self, resource, rscpath, conf):

        for cat in conf.values():
            for param in cat.values():
                resource.setdefault(cat.name, {})[param.name] = param.svalue

        with open(rscpath, 'wb') as handle:

            dump(resource, handle)


class FileConfDriverTest(ConfDriverTest):
    """Configuration Manager unittest class."""

    __driverclass__ = TestFileConfDriver

    def setUp(self):

        super(FileConfDriverTest, self).setUp()

        last_conf_dir = CONF_DIRS[-1]

        for path in self.paths:

            rscpath = join(last_conf_dir, path)

            with open(rscpath, 'w+') as _:
                pass

    def tearDown(self):

        for path in self.paths:

            for rscpath in self.driver.rscpaths(path):

                if exists(rscpath):
                    remove(rscpath)


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-

# --------------------------------------------------------------------
# The MIT License (MIT)
#
# Copyright (c) 2014 Jonathan Labéjof <jonathan.labejof@gmail.com>
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
# --------------------------------------------------------------------

"""Configuration definition objects."""

__all__ = ['ModelElement', 'CompositeModelElement']

from b3j0f.utils.version import OrderedDict


class ModelElement(object):
    """Base configuration elementParameter.

    Use a name and a provide a python value in ordr to ease serialization.

    A model element uses a local flag in order to indicates if it has been
    created with the upper composite model element or if it has been added after
    updating."""

    class Error(Exception):
        """Handle ModelElement errors."""

    __slots__ = ()

    def clean(self):
        """Clean this model element."""

        raise NotImplementedError()

    def copy(self, cleaned=False, *args, **kwargs):
        """Copy this model element and contained elements if they exist.

        :param bool cleaned: if True (default False) copy this element without
            parameter values/svalues.
        """

        result = type(self)(*args, **kwargs)

        if cleaned:
            result.clean()

        return result

    def __eq__(self, other):

        result = isinstance(other, self.__class__)

        if result:

            result = True
            for slot in self.__slots__:

                selfattr = getattr(self, slot)
                otherattr = getattr(other, slot)

                result = selfattr == otherattr

                if not result:
                    break

        return result

    def __ne__(self, other):

        return not self.__eq__(other)

    def __repr__(self):

        slotsrepr = ''

        for __slot__ in self.__slots__:

            if __slot__.startswith('_'):
                if hasattr(self, __slot__):
                    __slot__ = __slot__[1:]

                else:
                    continue

            slotattr = getattr(self, __slot__)
            slotsrepr = '{0}{1}={2}, '.format(slotsrepr, __slot__, slotattr)

        if slotsrepr:
            slotsrepr = slotsrepr[:-2]

        result = '{0}[{1}]'.format(type(self).__name__, slotsrepr)

        return result

    def update(self, other, copy=True, cleaned=False, *args, **kwargs):
        """Update this element related to other element.

        :param other: same type than this.
        :param bool copy: copy other before update attributes.
        :param bool cleaned: do a cleaned copy.
        :param tuple args: copy args.
        :param dict kwargs: copy kwargs.
        :return: this"""

        if other:  # dirty hack for python2.6

            if isinstance(other, self.__class__):

                if copy:
                    other = other.copy(cleaned=cleaned, *args, **kwargs)

                for slot in self.__slots__:
                    oattr = getattr(other, slot)

                    if oattr is not None:
                        setattr(self, slot, oattr)

            else:
                raise TypeError(
                    'Wrong element to update with {0}: {1}'.format(self, other)
                )

        return self


class CompositeModelElement(ModelElement, OrderedDict):
    """Model element composed of model elements."""

    __contenttype__ = ModelElement  #: content type.

    __slots__ = ModelElement.__slots__

    def __init__(self, melts=None):
        """
        :param tuple melts: model elements to add.
        """

        super(CompositeModelElement, self).__init__({})

        if melts is not None:
            for melt in melts:
                self[melt.name] = melt

    def __deepcopy__(self, _):

        return self.copy()

    def __getattr__(self, key):
        """Try to delegate key attribute to content name."""
        result = None

        if key in self.__slots__:
            result = super(CompositeModelElement, self).__getattribute__(key)

        else:
            try:
                result = self[key]

            except KeyError:
                raise AttributeError(
                    '\'{0}\' object has no attribute \'{1}\''.format(
                        self.__class__, key
                    )
                )

        return result

    def __i(self, other, func):
        """Process input other with input func.

        :param ModelElement(s) other: other ModelElement(s) to process.
        :param func: function to apply on a ModelElement.
        :return: self.
        :raise: TypeError if other is not a ModelElement(s)."""

        if isinstance(other, type(self)):
            other = tuple(other)

        elif isinstance(other, self.__contenttype__):
            other = (other, )

        for melt in list(other):
            if not isinstance(melt, self.__contenttype__):
                raise TypeError('Wrong element {0}'.format(melt))

            else:
                func(melt)

        return self

    def __iadd__(self, other):
        """Put ModelElement(s) in this content.

        :param ModelElement(s) other: other element(s) to put to this.
        :return: self.
        :raise: TypeError if other is not a ModelElement(s)."""

        return self.__i(
            other=other, func=lambda melt: self.__setitem__(melt.name, melt)
        )

    def __isub__(self, other):
        """Remove a set of ModelElement in this content.

        :param ModelElement(s) other: ModelElement(s) to remove from this.
        :return: self.
        :raise: TypeError if other is not a ModelElement(s)."""

        return self.__i(other=other, func=lambda melt: self.pop(melt.name))

    def __ixor__(self, other):
        """Put other element(s) which are not registered in this.

        :param ModelElement(s) other: element(s) to put in this content.
        :return: self.
        :raise: TypeError if other is not a ModelElement(s)."""

        return self.__i(
            other=other,
            func=lambda melt:
            melt.name not in self and self.__setitem__(melt.name, melt)
        )

    def __repr__(self):

        result = super(CompositeModelElement, self).__repr__()

        result += OrderedDict.__repr__(self)[len(self.__class__.__name__) + 2: -2]

        return result

    def clean(self, *args, **kwargs):

        for content in list(self.values()):

            content.clean(*args, **kwargs)

    def copy(self, cleaned=False, *args, **kwargs):

        for slot in self.__slots__:
            if slot not in kwargs:
                attr = getattr(self, slot)
                kwargs[slot] = attr

        melts = list(melt.copy(cleaned=cleaned) for melt in self.values())

        result = super(CompositeModelElement, self).copy(melts=melts, *args, **kwargs)

        return result

    def update(self, other, copy=True, cleaned=False, *args, **kwargs):
        """Update this composite model element with other element content.

        :param other: element to update with this. Must be the same type of this
            or this __contenttype__.
        :param bool copy: copy other before updating.
        :param bool cleaned: in addition to the copy request, do a cleaned copy.
        """

        super(CompositeModelElement, self).update(
            other, copy=copy, cleaned=cleaned, *args, **kwargs
        )

        if other:  # dirty hack for python2.6
            contents = []

            if isinstance(other, self.__class__):
                contents = list(other.values())

            elif isinstance(other, self.__contenttype__):
                contents = [other]

            else:
                raise TypeError(
                    'Wrong element to update with {0}: {1}'.format(self, other)
                )

            for content in contents:

                selfcontent = self.get(content.name)

                if selfcontent is None:

                    if copy:
                        content = content.copy(cleaned=cleaned, local=False)
                    self[content.name] = content

                else:
                    selfcontent.update(
                        content, copy=copy, cleaned=cleaned, *args, **kwargs
                    )

    @property
    def params(self):
        """Get set of parameters by names.

        :rtype: dict"""

        result = {}

        for content in list(self.values()):

            if isinstance(content, CompositeModelElement):
                result.update(content.params)

            else:
                result[content.name] = content

        return result



# -*- coding: utf-8 -*-

# --------------------------------------------------------------------
# The MIT License (MIT)
#
# Copyright (c) 2014 Jonathan Labéjof <jonathan.labejof@gmail.com>
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
# --------------------------------------------------------------------

"""XML configuration driver."""

from __future__ import absolute_import

__all__ = ['XMLConfDriver']

from b3j0f.utils.version import PY2

from xml.etree.ElementTree import fromstring, Element, ElementTree

from .base import ConfDriver
from ..model.param import Parameter


class XMLConfDriver(ConfDriver):
    """Manage xml resource configuration."""

    CONFIGURATION = 'configuration'
    PARAMETER = 'parameter'
    CATEGORY = 'category'

    def rscpaths(self, path):

        return [path]

    def resource(self):

        return ElementTree(Element(XMLConfDriver.CONFIGURATION))

    def _pathresource(self, rscpath):

        result = fromstring(rscpath)

        return result

    def _cnames(self, resource):

        result = list(
            cat.get('name') for cat in resource.findall(XMLConfDriver.CATEGORY)
        )

        return result

    def _params(self, resource, cname):

        result = []

        if PY2:
            ecats = resource.findall(XMLConfDriver.CATEGORY)

            ecat = None

            for ecat in ecats:
                if ecat.get('name') == cname:
                    break

            else:
                ecat = None

            if ecat is not None:
                eparams = ecat.findall(XMLConfDriver.PARAMETER)

                result = list(Parameter(**eparam.attrib) for eparam in eparams)

        else:
            query = '{0}[@name=\'{1}\']/{2}'.format(
                XMLConfDriver.CATEGORY,
                cname,
                XMLConfDriver.PARAMETER
            )

            result = list(
                Parameter(**param.attrib) for param in resource.findall(query)
            )

        return result

    def _setconf(self, conf, resource, rscpath):

        for cat in conf.values():

            cquery = XMLConfDriver.CATEGORY

            ecat = None

            if PY2:
                ecats = resource.findall(cquery)
                for ecat in ecats:
                    if ecat.get('name') == 'cat.name':
                        break

                else:
                    ecat = None

            else:
                cquery += '[@name=\'{0}\']'.format(cat.name)
                ecat = resource.find(cquery)

            if ecat is None:
                ecat = Element(XMLConfDriver.CATEGORY, name=cat.name)
                resource.getroot().append(ecat)

            for param in cat.values():

                pquery = XMLConfDriver.PARAMETER

                eparam = None

                if PY2:
                    eparams = ecat.findall(pquery)
                    for eparam in eparams:
                        if eparam.get('name') == param.name:
                            break

                    else:
                        eparam = None

                else:
                    cquery += '[@name=\'{0}\']'.format(param.name)

                    eparam = ecat.find(pquery)

                if eparam is None:
                    eparam = Element(XMLConfDriver.PARAMETER, name=param.name)
                    ecat.append(eparam)

                if param.svalue is not None:
                    eparam.set('svalue', param.svalue)

        return resource.getroot().text



__all__ = [
    'ExprResolver', 'ResolverRegistry', 'names', 'resolve', 'register',
    'loadresolvers', 'defaultname', 'resolvejs', 'resolvepy'
]

from .base import ExprResolver
from .registry import (
    ResolverRegistry, names, resolve, register, loadresolvers, defaultname
)
from .lang import resolvejs, resolvepy



# -*- coding: utf-8 -*-

# --------------------------------------------------------------------
# The MIT License (MIT)
#
# Copyright (c) 2014 Jonathan Labéjof <jonathan.labejof@gmail.com>
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
# --------------------------------------------------------------------

"""model.cat module."""

__all__ = ['Category']

from .base import CompositeModelElement
from .param import Parameter


class Category(CompositeModelElement):
    """Parameter category which contains a dictionary of params."""

    __contenttype__ = Parameter  #: content type.

    __slots__ = ('name', 'local') + CompositeModelElement.__slots__

    def __init__(self, name, local=True, *args, **kwargs):
        """
        :param str name: category name to use.
        """

        super(Category, self).__init__(*args, **kwargs)

        self.name = name
        self.local = local

    def getparams(self, param):
        """Get parameters which match with input param.

        :param Parameter param: parameter to compare with this parameters.
        :rtype: list
        """

        return list(cparam for cparam in self.values() if cparam == param)

    def copy(self, cleaned=False, name=None, *args, **kwargs):

        if name is None:
            name = self.name

        return super(Category, self).copy(name=name, *args, **kwargs)


def category(name, *params):
    """Quick instanciation of category with parameteres."""

    return Category(name=name, melts=params)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# --------------------------------------------------------------------
# The MIT License (MIT)
#
# Copyright (c) 2014 Jonathan Labéjof <jonathan.labejof@gmail.com>
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
# --------------------------------------------------------------------

"""Configuration UTs."""

from unittest import main

from b3j0f.utils.ut import UTCase

from ..conf import Configuration, configuration
from ..cat import Category, category
from ..param import Parameter


class ConfigurationTest(UTCase):
    """Test exprparser."""

    def setUp(self):

        self.count = 5

        self.conf = Configuration()

        for count in range(self.count):

            cat = Category('{0}'.format(count))
            self.conf += cat

            for index in range(count):

                param = Parameter(
                    name='p{0}{1}'.format(count, index),
                    value=count + index
                )

                if (index + count) % 2 == 0:  # add errors
                    param._error = True
                    param.value = None

                cat += param

    def test_resolve(self):
        """Test the method resolve."""

        for cat in self.conf.values():
            for param in cat.values():
                param.svalue = '=1'

        self.conf.resolve()

        for cat in self.conf.values():
            for param in cat.values():
                self.assertEqual(param.value, 1)

    def test_params(self):
        """Test the params property."""

        params = self.conf.params

        errors = list(param for param in params.values() if param.error)
        self.assertEqual(len(errors), (self.count * (self.count + 1) // 8) + 1)

    def test_pvalue(self):
        """Test the method pvalue."""

        for cat in self.conf.values():

            for param in cat.values():

                value = param.value

                if value is None:
                    continue

                pvalue = self.conf.param(pname=param.name).value

                self.assertEqual(value, pvalue)

                pvalue = self.conf.param(pname=param.name, cname=cat.name).value

                self.assertEqual(value, pvalue)


if __name__ == '__main__':
    main()
